<?php

namespace Drupal\paragraph_blocks;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Path\AliasManager;
use Drupal\Core\Path\AliasManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Class HcpCoreManager.
 */
class ParagraphBlocksEntityManager implements ContainerInjectionInterface {

  /**
   * The path alias manager.
   *
   * @var \Drupal\Core\Path\AliasManagerInterface
   */
  protected $aliasManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /*
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * HcpCoreManager constructor.
   */
  public function __construct(AliasManagerInterface $alias_manager, EntityTypeManagerInterface $entity_type_manager, RequestStack $request_stack, ConfigFactoryInterface $config_factory) {
    $this->aliasManager = $alias_manager;
    $this->entityTypeManager = $entity_type_manager;
    $this->requestStack = $request_stack;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('path.alias_manager'),
      $container->get('entity_type.manager'),
      $container->get('request_stack'),
      $container->get('config.factory')
    );
  }

  /**
   * Return the entity of the path.
   *
   * @param string $path
   *   (optional) The path to lookup, defaults to the current path.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The entity of the current path, or null if none.
   *
   * @todo: This implements something similar to menu_get_object() which was
   * likely removed intentionally, so is this a bad idea?
   */
  public function getEntity($path = NULL) {
    // Get the current path if none is specified.
    if (!$path) {
      $path = $this->requestStack->getCurrentRequest()->getRequestUri();
    }
    if ($path == '/') {
      $path = $this->configFactory->get('system.site')->get('page.front');
      // Remove leading slash.
      $path = substr($path, 1);
    }

    // Convert the specified path to an internal path.
    $source_path = $this->aliasManager->getPathByAlias($path);

    // Check if this the entity path.
    // @todo: this makes assumption that are possibly not valid. For example, is
    // the entity path always "/entity_type_id/entity_id"?
    $parts = explode('/', trim($source_path, '/'));
    if (count($parts) >= 2) {
      list($entity_type_id, $entity_id) = $parts;
      // Check that the path is for an entity.
      /** @var \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager */
      $entity_types = $this->entityTypeManager->getDefinitions();
      if (is_numeric($entity_id) && isset($entity_types[$entity_type_id])) {
        /** @var \Drupal\Core\Entity\EntityStorageInterface $entity_storage */
        $entity_storage = $this->entityTypeManager->getStorage($entity_type_id);
        if ($entity_storage) {
          // Load and return the entity.
          return $entity_storage->load($entity_id);
        }
      }
    }
    return NULL;
  }

  /**
   * Return the entity of the referer, useful when called via AJAX.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The entity of the referer, or null if none.
   */
  public function getRefererEntity() {
    $referer = $this->requestStack->getCurrentRequest()->server->get('HTTP_REFERER');
    if ($referer) {
      $path = parse_url($referer, PHP_URL_PATH);
      if ($path) {
        return $this->getEntity($path);
      }
    }
    return NULL;
  }

  /**
   * Returns the plugin's paragraph title.
   *
   * @param string $plugin_id
   *   The plugin id.
   *
   * @return string
   *   The paragraph title.
   */
  public function getTitle($plugin_id) {
    $plugin_parts = explode(':', $plugin_id);
    if (count($plugin_parts) < 4) {
      return NULL;
    }
    list($plugin_type_id, , $plugin_field_name, $plugin_field_delta) = $plugin_parts;
    if ($plugin_type_id != $this->pluginTypeId) {
      return NULL;
    }

    // Return if this plugin should be removed from the list.
    if (!$this->entity || $plugin_field_delta >= $this->entity->get($plugin_field_name)->count()) {
      return FALSE;
    }

    // Get the referenced paragraph.
    /** @var \Drupal\paragraphs\Entity\Paragraph $paragraph */
    $paragraph = $this->entity->get($plugin_field_name)->referencedEntities()[$plugin_field_delta];

    // Change the label to match admin_label from the referenced paragraph.
    return str_replace('@label', $paragraph->getSummary(), $this->labelFormat);
  }

}
